﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.IO;
using System.Management.Automation;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.PlatformUI;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.Win32;
using Task = System.Threading.Tasks.Task;

namespace CPABuildEvents
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    /// </summary>
    /// <remarks>
    /// <para>
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the
    /// IVsPackage interface and uses the registration attributes defined in the framework to
    /// register itself and its components with the shell. These attributes tell the pkgdef creation
    /// utility what data to put into .pkgdef file.
    /// </para>
    /// <para>
    /// To get loaded into VS, the package must be referred by &lt;Asset Type="Microsoft.VisualStudio.VsPackage" ...&gt; in .vsixmanifest file.
    /// </para>
    /// </remarks>
    [PackageRegistration(UseManagedResourcesOnly = true, AllowsBackgroundLoading = true)]
    [InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)] // Info on this package for Help/About
    [ProvideAutoLoad(UIContextGuids80.SolutionExists, PackageAutoLoadFlags.BackgroundLoad)]
    [Guid(BuildEventsPackage.PackageGuidString)]
    [SuppressMessage("StyleCop.CSharp.DocumentationRules", "SA1650:ElementDocumentationMustBeSpelledCorrectly", Justification = "pkgdef, VS and vsixmanifest are valid VS terms")]
    public sealed class BuildEventsPackage : AsyncPackage
    {
        /// <summary>
        /// VSPackage1 GUID string.
        /// </summary>
        public const string PackageGuidString = "90d6e709-5902-4603-8f56-3b4cdaaa784f";

        private readonly Lazy<DTE2> _dte;
        private readonly Lazy<Events> _events;
        private readonly Lazy<BuildEvents> _buildEvents;
        private readonly Lazy<IVsOutputWindow> _outWindow;
        private readonly Lazy<IVsOutputWindowPane> _buildPane;

        /// <summary>
        /// Initializes a new instance of the <see cref="BuildEventsPackage"/> class.
        /// </summary>
        public BuildEventsPackage()
        {
            // Inside this method you can place any initialization code that does not require
            // any Visual Studio service because at this point the package object is created but
            // not sited yet inside Visual Studio environment. The place to do all the other
            // initialization is the Initialize method.
            _dte = new Lazy<DTE2>(() => ServiceProvider.GlobalProvider.GetService(typeof(DTE)) as DTE2);
            _events = new Lazy<Events>(() => _dte.Value.Events);
            _buildEvents = new Lazy<BuildEvents>(() => _events.Value.BuildEvents);
            _outWindow = new Lazy<IVsOutputWindow>(() => GetGlobalService(typeof(SVsOutputWindow)) as IVsOutputWindow);
            _buildPane = new Lazy<IVsOutputWindowPane>(() =>
            {
                _outWindow.Value.GetPane(VSConstants.GUID_BuildOutputWindowPane, out var pane);
                return pane;
            });
        }

        #region Package Members

        /// <summary>
        /// Initialization of the package; this method is called right after the package is sited, so this is the place
        /// where you can put all the initialization code that rely on services provided by VisualStudio.
        /// </summary>
        /// <param name="cancellationToken">A cancellation token to monitor for initialization cancellation, which can occur when VS is shutting down.</param>
        /// <param name="progress">A provider for progress updates.</param>
        /// <returns>A task representing the async work of package initialization, or an already completed task if there is none. Do not return null from this method.</returns>
        protected override async Task InitializeAsync(CancellationToken cancellationToken, IProgress<ServiceProgressData> progress)
        {
            // When initialized asynchronously, the current thread may be a background thread at this point.
            // Do any initialization that requires the UI thread after switching to the UI thread.
            await JoinableTaskFactory.SwitchToMainThreadAsync(cancellationToken);

            _buildEvents.Value.OnBuildBegin += DoBuildBegin;
            _buildEvents.Value.OnBuildDone += DoBuildEnd;
        }

        private void DoBuildEnd(vsBuildScope Scope, vsBuildAction Action)
        {
            ThreadHelper.ThrowIfNotOnUIThread();
            var solution = _dte.Value.Solution;
            if (solution != null)
            {
                var postBuildScript = Path.Combine(Path.GetDirectoryName(solution.FullName), "_postbuild.ps1");
                if (File.Exists(postBuildScript))
                {
                    new RunPSScript(postBuildScript, _buildPane.Value).RunScript();
                }
            }
        }

        private void DoBuildBegin(vsBuildScope Scope, vsBuildAction Action)
        {
            ThreadHelper.ThrowIfNotOnUIThread();
            var solution = _dte.Value.Solution;
            if (solution != null)
            {
                var preBuildScript = Path.Combine(Path.GetDirectoryName(solution.FullName), "_prebuild.ps1");
                if (File.Exists(preBuildScript))
                {
                    new RunPSScript(preBuildScript, _buildPane.Value).RunScript();
                }
            }
        }

        #endregion
    }
}
